@@ -28,6 +28,7 @@
 #include <vector>
 
 #include <gpgme++/global.h>
+#include <gpgme++/configuration.h>
 
 namespace GpgME
 {
@@ -59,6 +60,8 @@ public:
        */
     std::vector<GpgME::Key> getEncryptionKeys (const std::vector<std::string> &recipients,
                                                GpgME::Protocol proto) const;
+    std::vector<GpgME::Key> getEncryptionKeys (const std::string &recipient,
+                                               GpgME::Protocol proto) const;
 
     /* Start a key location in a background thread filling
        the key cache.
@@ -136,6 +139,16 @@ public:
        true on success. */
     static bool import_pgp_key_data(const GpgME::Data &data);
 
+    /* Get the cached GnuPG configuration that is only loaded
+       once on start for quick checks. */
+    const std::vector<GpgME::Configuration::Component> get_cached_config () const;
+
+    /* Check the cached configuration if operations might
+       use online calls. */
+    bool protocolIsOnline (GpgME::Protocol proto) const;
+
+    bool useTofu () const;
+
     // Internal for thread
     void setSmimeKey(const std::string &mbox, const GpgME::Key &key);
     void setPgpKey(const std::string &mbox, const GpgME::Key &key);
@@ -145,6 +158,7 @@ public:
     void onAddrBookImportJobDone (const std::string &fpr,
                                   const std::vector<std::string> &result_fprs,
                                   GpgME::Protocol proto);
+    void setConfig(const std::vector<GpgME::Configuration::Component> & comp);
 
 private:
 
