@@ -33,16 +33,19 @@
 
 #include <sstream>
 
+#include "cpphelp.h"
+
 #ifdef HAVE_W32_SYSTEM
 #include "common.h"
 /* We use UTF-8 internally. */
 #undef _
 # define _(a) utf8_gettext (a)
 #else
+# undef S_
+# define S_(a) std::string (a)
 # define _(a) a
 #endif
 
-
 const char decrypt_template_html[] = {
 "<html><head></head><body>"
 "<table border=\"0\" width=\"100%%\" cellspacing=\"1\" cellpadding=\"1\" bgcolor=\"#0069cc\">"
@@ -116,7 +119,6 @@ ParseController::ParseController(LPSTREAM instream, msgtype_t type):
     m_type (type),
     m_block_html (false),
     m_second_pass (false)
-
 {
   TSTART;
   memdbg_ctor ("ParseController");
@@ -368,6 +370,16 @@ ParseController::parse(bool offline)
       verify = true;
       decrypt = false;
     }
+  else if (m_second_pass && inputType != Data::Type::PGPEncrypted &&
+           /* For CMS there might be combined messages when gnupg supports
+            * this and which is not caught below. */
+           inputType != Data::Type::CMSOther &&
+           inputType != Data::Type::CMSEncrypted)
+    {
+      log_dbg ("Second pass for message. Only doing verify.");
+      verify = true;
+      decrypt = false;
+    }
   else
     {
       operation_for_type (m_type, &decrypt, &verify);
@@ -430,6 +442,13 @@ ParseController::parse(bool offline)
       ctx->setOffline (true);
     }
 
+  if (m_second_pass)
+    {
+      // Always use a fresh output on second pass
+      delete m_outputprovider;
+      m_outputprovider = new MimeDataProvider (expect_no_mime (m_type));
+    }
+
   Data output (m_outputprovider);
   log_debug ("%s:%s:%p decrypt: %i verify: %i with protocol: %s sender: %s type: %i",
              SRCNAME, __func__, this,
@@ -454,6 +473,7 @@ ParseController::parse(bool offline)
           output.type() == Data::Type::PGPSigned)
         {
           TRACEPOINT;
+          log_dbg ("Did not have combined result parsing output.");
           /* There is a signature in the output. So we have
              to verify it now as an extra step. */
           input = Data (m_outputprovider);
@@ -469,10 +489,23 @@ ParseController::parse(bool offline)
           verify = false;
         }
       TRACEPOINT;
-      if (m_decrypt_result.error () || m_decrypt_result.isNull () ||
-          m_decrypt_result.error ().isCanceled ())
+      const auto err = m_decrypt_result.error ();
+      if (err || m_decrypt_result.isNull () ||
+          err.isCanceled ())
         {
           m_error = format_error (m_decrypt_result, protocol);
+          if (err.code () == GPG_ERR_NO_DATA)
+            {
+              m_error += std::string ("<pre>") +
+                S_ ("If this data does not look like an encrypted message\n"
+                    "please see the debug tab in the options on how to report\n"
+                    "this for further improvement.") + std::string ("\n\n") +
+                asprintf_s (_("Debug information: Message type: %i Data type: %i"),
+                               m_type, inputType) + std::string ("\n\n") +
+                S_ ("The interpreted data was:") + std::string ("\n</pre><pre>");
+              m_error += input.toString ();
+              m_error += "</pre>";
+            }
         }
     }
   if (verify)
